package com.gic.haoban.manage.service.service.out.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.clerk.api.dto.ClerkQwDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.commons.util.*;
import com.gic.haoban.app.customer.service.api.service.InnerApiService;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.AppPageType;
import com.gic.haoban.manage.api.enums.NoticeMessageTypeEnum;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.api.enums.WxEditType;
import com.gic.haoban.manage.api.service.*;
import com.gic.haoban.manage.api.util.notify.NoticeMessageUtil;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.*;
import com.gic.haoban.manage.service.service.*;
import com.gic.mq.sdk.GicMQClient;
import com.gic.thirdparty.api.dto.PicUploadResDTO;
import com.gic.thirdparty.api.service.QQCloudPicService;
import com.gic.wechat.api.dto.qywx.ItemDTO;
import com.gic.wechat.api.dto.qywx.QywxXcxSendMessageDTO;
import com.gic.wechat.api.service.qywx.QywxDepartmentApiService;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.google.common.collect.Sets;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 接收企业微信或者gic推送消息进行处理
 * Created by tgs on 2020/2/20.
 */
@Service("messageApiService")
public class MessageApiServiceImpl implements MessageApiService {
    private static final Logger log = LogManager.getLogger(MessageApiServiceImpl.class);
    private final String CONTACT_APP = "contactSuiteId";
    @Autowired
    private DepartmentService departmentService;
    @Autowired
    private DepartmentApiService departmentApiService;
    @Autowired
    private WxEnterpriseService enterpriseService;
    @Autowired
    private StaffService staffService;
    @Autowired
    private StaffDepartmentRelatedService staffDepartmentRelatedService;
    @Autowired
    private StaffApiService staffApiService;
    @Autowired
    private Config config;
    @Autowired
    private QywxDepartmentApiService qywxDepartmentApiService;

    @Autowired
    private QQCloudPicService qqCloudPicService;

    @Autowired
    private InnerApiService innerApiService;

    @Autowired
    private StaffClerkRelationApiService staffClerkRelationApiService;

    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;

    @Autowired
    private WxEnterpriseService wxEnterpriseService;
    @Autowired
    private StaffDepartmentRelatedApiService staffDepartmentRelatedApiService;
    @Autowired
    private WxEnterpriseRelatedService wxEnterpriseRelatedService;
    @Autowired
    private ClerkService clerkService;
    @Autowired
    private SecretSettingService secretSettingService;
    @Autowired
    private WxApplicationService wxApplicationService;

    @Override
    public void gicMessageReceive(GicMessageDTO gicMessageDTO) {
        if (true) {
            log.info("好办3.0去除消息监听");
            return;
        }
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            log.info("发送消息到dealGicMessageMq，{}", JSON.toJSONString(gicMessageDTO));
            clientInstance.sendMessage("dealGicMessageMq", JSON.toJSONString(gicMessageDTO));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void wxMessageReceive(QywxCallBackDTO qywxCallBackDTO) {
        GicMQClient clientInstance = GICMQClientUtil.getClientInstance();
        try {
            log.info("发送消息到dealWxMessageMq，{}", JSON.toJSONString(qywxCallBackDTO));
            clientInstance.sendMessage("dealWxMessageMq", JSON.toJSONString(qywxCallBackDTO));
        } catch (Exception e) {
            log.error("发送失败：{}", e.getMessage());
        }
    }

    @Override
    public void dealWxMessage(String param) {
        QywxCallBackDTO dto = JSON.parseObject(param, QywxCallBackDTO.class);
        log.info("【成员同步回调处理】dto={}", JSON.toJSONString(param));
        TabHaobanWxApplication wxApplication = wxApplicationService.selectByCorpId(dto.getAuthCorpId());
        if (Objects.isNull(wxApplication)) {
            log.info("通过corpId:【{}】未查询到授权企业,无法处理", dto.getAuthCorpId());
            return;
        }
        String wxEnterpriseId = wxApplication.getWxEnterpriseId();
        // 校验适用企业-定制
        boolean suitWxEnterpriseIid = checkSuitEnterpriseId(wxEnterpriseId);
        if (suitWxEnterpriseIid) {
            log.info("成员同步定制化开始");
            // 处理定制-目前鸿星尔克用
            boolean checkErkeDeal = checkErkeDeal(dto);
            if (!checkErkeDeal) {
                log.info("不符合鸿星尔克需求,返回");
                return;
            }
            log.info("微信企业id:{}, 不适用", wxEnterpriseId);
            List<TabHaobanWxEnterpriseRelated> relatedList = wxEnterpriseRelatedService.getByWxEnterpriseId(wxEnterpriseId);
            if (CollectionUtils.isEmpty(relatedList)) {
                log.info("未查询到企业关联企业 wxEnt");
                return;
            }
            ServiceResponse response = staffApiService.getWxSaveNew(dto.getUserid(), wxEnterpriseId);
            if (response.getCode() != 1) {
                log.info("添加企业员工失败:{}", response.getMessage());
                return;
            }
            SecretSettingDTO secretSetting = secretSettingService.getSecretSetting(wxEnterpriseId, SecretTypeEnum.HAOBAN_HELP.getVal());
            if (null == secretSetting || secretSetting.getCheckFlag() == 0) {
                log.info("没有配置secret：{}", JSONObject.toJSONString(wxApplication));
                return;
            }
            // 这里将员工姓名改一下, 代开发返回的 id 和 name 一样
            if (Objects.equals(dto.getUserid(), dto.getUserName())) {
                dto.setUserName((String) response.getResult());
            }
            // 取第一个企业的id
            doDealUser(dto, relatedList.get(0).getEnterpriseId(), secretSetting.getSecretVal());
            log.info("成员同步定制化结束");
        } else {
            dealUser(dto);
        }
    }

    /**
     * desc: 鸿星尔克定制需求校验-后期可扩展为配置形式
     *
     * @param dto dto
     * @return : {@link boolean}
     * @author : YongEn
     * @date : 2022/1/25
     */
    private boolean checkErkeDeal(QywxCallBackDTO dto) {
        if (Objects.isNull(dto) || StringUtils.isEmpty(dto.getUserid())) {
            return false;
        }
        if (!"create_user".equals(dto.getChangeType())) {
            log.info("只处理新增用户, 非 create_user 类型不做处理 当前传入类型:{}", dto.getChangeType());
            return false;
        }
        String userid = dto.getUserid();
        // 只处理5开头的-不是5开头不同步
        return userid.startsWith("5");
    }

    private boolean checkSuitEnterpriseId(String wxEnterpriseId) {
        Set<String> wxSet = Sets.newHashSet(
                "ac451a350eed4a96958dd5c7377f8bd9",
                // 鸿星尔克
                "e034e2bd5d0b44d3bf553c9f8204094c");
        return wxSet.contains(wxEnterpriseId);
    }

    private void doDealUser(QywxCallBackDTO dto, String enterpriseId, String secret) {
        ClerkQwDTO clerkQwDTO = new ClerkQwDTO();
        clerkQwDTO.setEnterpriseId(enterpriseId);
        String mainDeptId = dto.getMainDepartment();
        List<com.gic.wechat.api.dto.qywx.DepartmentDTO> departments = qywxDepartmentApiService.listSelfDepartment(dto.getAuthCorpId(), secret, Integer.valueOf(mainDeptId));
        if (CollectionUtils.isEmpty(departments)) {
            log.info("通过主部门查询部门列表为空 mainDeptId:{}", mainDeptId);
            return;
        }
        Map<Integer, com.gic.wechat.api.dto.qywx.DepartmentDTO> departmentIdMap = departments.stream()
                .collect(Collectors.toMap(com.gic.wechat.api.dto.qywx.DepartmentDTO::getId, Function.identity(), (a, b) -> a));
        com.gic.wechat.api.dto.qywx.DepartmentDTO mainDept = departmentIdMap.get(Integer.valueOf(mainDeptId));
        clerkQwDTO.setStoreName(mainDept.getName());
        clerkQwDTO.setClerkCode(dto.getUserid());
        clerkQwDTO.setClerkName(dto.getUserName());
        clerkQwDTO.setClerkType(dto.getIsLeaderInDept()[0]);
        clerkQwDTO.setClerkGener(dto.getGender());
        clerkQwDTO.setPhoneNumber(dto.getMobile());
        clerkQwDTO.setNationcode(dto.getTelephone());
        // 调用换导购接口逻辑
        clerkService.syncClerkForQw(clerkQwDTO);
    }

    /**
     * 处理企业通讯录成员
     *
     * @param qywxCallBackDTO
     */
    private void dealUser(QywxCallBackDTO qywxCallBackDTO) {
        String userid = qywxCallBackDTO.getUserid();
        log.info("成员同步开始：userid:{}", userid);
        TabHaobanWxEnterprise enterprise = this.enterpriseService.getEnterpriseBycorpId(qywxCallBackDTO.getAuthCorpId());
        if (enterprise == null) {
            log.info("授权企业不存在，{}", qywxCallBackDTO.getAuthCorpId());
            return;
        }
        String wxEnterpriseId = enterprise.getWxEnterpriseId();
        String changeType = qywxCallBackDTO.getChangeType();

        if (changeType.equals(WxEditType.ADDUSER.getCode())) {
            log.info("成员同步新增，userid:{}", userid);
            //新增
            this.staffApiService.wxGetAdd(userid, wxEnterpriseId);
        } else if (changeType.equals(WxEditType.UPDATEUSER.getCode())) {
            log.info("成员同步修改，userid:{}", userid);
            //修改
            TabHaobanStaff oldStaff = this.staffService.selectByUserIdAndEnterpriseId(userid, wxEnterpriseId);
            if (oldStaff == null) {
                log.error("成员同步更新失败，无历史员工数据：userid:{},wxEnterpriseId:{}", userid, wxEnterpriseId);
                return;
            }
            TabHaobanStaff staff = new TabHaobanStaff();
            staff.setStaffId(oldStaff.getStaffId());
            staff.setPhoneNumber(qywxCallBackDTO.getMobile());
            staff.setNickName(qywxCallBackDTO.getAlias());
            staff.setSex(qywxCallBackDTO.getGender());
            staff.setStaffName(qywxCallBackDTO.getUserName());
            staff.setWxUserId(userid);
            staff.setPostion(qywxCallBackDTO.getPosition());
            if (StringUtils.isNotBlank(qywxCallBackDTO.getAvatar())) {
                staff.setHeadImg(changeHeaderImageUrl(qywxCallBackDTO.getAvatar()));
            }
            staff.setNationCode("86");
            //激活状态
            if (qywxCallBackDTO.getStatus() != null && qywxCallBackDTO.getStatus() == 1) {
                staff.setActiveFlag(1);
            }
            String[] departArr = qywxCallBackDTO.getDepartment();
            StringBuilder departmentIds = new StringBuilder();
            //部门修改了
            if (departArr != null) {
                for (String s : departArr) {
                    TabHaobanDepartment department = this.departmentService.getByWxId(s, wxEnterpriseId);
                    if (department == null) {
                        log.info("成员同步，部门不存在");
                        continue;
                    }
                    departmentIds.append(department.getDepartmentId()).append(",");
                }
                departmentIds = new StringBuilder(departmentIds.substring(0, departmentIds.length() - 1));
            } else {
                //部门没有修改
                List<TabHaobanStaffDepartmentRelated> list = staffDepartmentRelatedService.listStaffDepartmentByStaffId(staff.getStaffId());
                for (TabHaobanStaffDepartmentRelated tabHaobanStaffDepartmentRelated : list) {
                    departmentIds.append(tabHaobanStaffDepartmentRelated.getDepartmentId()).append(",");
                }
                departmentIds = new StringBuilder(departmentIds.substring(0, departmentIds.length() - 1));
            }
            StaffDTO staffDTO = EntityUtil.changeEntityByJSON(StaffDTO.class, staff);
            staffApiService.staffEdit(staffDTO, departmentIds.toString());
        } else if (changeType.equals(WxEditType.DELETEUSER.getCode())) {
            log.info("成员同步删除，userid:{}", userid);
            //删除
            TabHaobanStaff oldStaff = this.staffService.selectByUserIdAndEnterpriseId(userid, wxEnterpriseId);
            if (oldStaff == null) {
                log.info("微信成员同步删除的门店不存在，{}", userid);
                return;
            }
            //销毁卡券
            innerApiService.delCardByStaffId(oldStaff.getStaffId());
            //删除导购好友关联关系表
            this.staffDepartmentRelatedService.delByUserid(userid);
            //删除员工表
            this.staffService.delByStaffId(oldStaff.getStaffId());
        }else {
            log.error("成员同步类型错误，changeType：{}",changeType);
        }
        log.error("成员同步结束");
    }

    private String changeHeaderImageUrl(String headImgUrl) {
        try {
            log.info("图片headImgUrl:{}",headImgUrl);
            if (StringUtils.isBlank(headImgUrl)) {
                return headImgUrl;
            }
            byte[] data = ImageUtil.getImgbyte(headImgUrl);
            PicUploadResDTO uploadPic = qqCloudPicService.uploadPic(GlobalVar.ctxPropertiesMap.get(GlobalInfo.QQPIC_KEY_ENTERPRISE), data);
            log.info("腾讯云万象优图返回" + JSON.toJSONString(uploadPic));
            return uploadPic.downloadUrl;
        } catch (Exception e) {
            log.info("上传腾讯云万象优图返回失败了：{}", e.getMessage());
            return headImgUrl;
        }
    }

    @Override
    public Boolean sendFaceMessage(String clerkId, String memberId, String memberName, String arrivalTime, String storeId) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("memberId", memberId);
        jsonObject.put("storeId", storeId);
        String data = jsonObject.toJSONString();
        String pageUrl = staffDepartmentRelatedApiService.getPageUrl(AppPageType.FACE_ARRIVAL.getCode(), data);
        StaffClerkRelationDTO relation = staffClerkRelationApiService.getByClerkId(clerkId);
        if (relation == null) {
            log.info("clerkId未绑定:clerkId={}", clerkId);
            return false;
        }
        WxEnterpriseDTO wxEnterprise = wxEnterpriseService.selectById(relation.getWxEnterpriseId());
        if (wxEnterprise == null) {
            log.info("企业为空");
            return false;
        }
        StaffDTO staffDTO = staffApiService.selectById(relation.getStaffId());
        String corpid = wxEnterprise.getCorpid();
        String wxUserId = "";
        if (corpid.length() > 20) {
            wxUserId = staffDTO.getWxOpenUseId();
        } else {
            wxUserId = staffDTO.getWxUserId();
        }
        QywxXcxSendMessageDTO messageDTO = new QywxXcxSendMessageDTO();
        Map<String, String> map = new HashMap<>();
        map.put("事件", "人脸匹配");
        map.put("会员", memberName);
        map.put("到店时间", arrivalTime);
        List<ItemDTO> items = getItemsList(map);
        ArrayList<String> list = new ArrayList<>();
        list.add(wxUserId);
        messageDTO.setAppid(config.getAppid());
        messageDTO.setUserIds(list);
        messageDTO.setPage(pageUrl);
        messageDTO.setTitle("我的顾客通知");
        messageDTO.setItems(items);
        return qywxSuiteApiService.sendMessage(corpid, config.getWxSuiteid(), messageDTO);
    }

    private List<ItemDTO> getItemsList(Map<String, String> map) {
        List<ItemDTO> items = new ArrayList();
        map.keySet().forEach(key -> {
            String value = map.get(key);
            ItemDTO dto = new ItemDTO();
            dto.setKey(key);
            dto.setValue(value);
            items.add(dto);
        });
        return items;
    }

    @Override
    public Boolean sendPerfectRemarkMessage(String clerkId, String memberId, String memberName, String customerTime, String orderId, String storeId) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("memberId", memberId);
        jsonObject.put("storeId", storeId);
        jsonObject.put("currentTab", 1);
        StaffClerkRelationDTO relation = staffClerkRelationApiService.getByClerkId(clerkId);
        if (relation == null) {
            log.info("clerkId未绑定:clerkId={}", clerkId);
            return false;
        }
        WxEnterpriseDTO wxEnterprise = wxEnterpriseService.selectById(relation.getWxEnterpriseId());
        if (wxEnterprise == null) {
            log.info("企业为空");
            return false;
        }
        StaffDTO staffDTO = staffApiService.selectById(relation.getStaffId());
        if (staffDTO == null) {
            return false;
        }
        Map<String, String> params = new HashMap<>();
        params.put("memberName", memberName);
        params.put("orderId", orderId);
        params.put("customerTime", customerTime);

        NoticeMessageUtil.sendNoticeMessage(relation.getEnterpriseId(), relation.getClerkId(), NoticeMessageTypeEnum.MEMBER_FACE.getType(), null, params, jsonObject);
        return true;

    }

    @Override
    public void sendWxMessage(String param) {
        log.info("发送企业微信消息：{}", param);
        if (StringUtils.isBlank(param)) {
            log.info("param参数为空");
            return;
        }
        JSONObject jsonObject = JSON.parseObject(param);
        String wxEnterpriseId = jsonObject.getString("wxEnterpriseId");
        String wxUserId = jsonObject.getString("wxUserId");
        String pageUrl = jsonObject.getString("pageUrl");
        String title = jsonObject.getString("title");
        String content = jsonObject.getString("content");
        String memberName = jsonObject.getString("memberName");
        WxEnterpriseDTO wxEnterprise = wxEnterpriseService.selectById(wxEnterpriseId);
        if (wxEnterprise == null) {
            log.info("企业为空");
            return;
        }
        QywxXcxSendMessageDTO messageDTO = new QywxXcxSendMessageDTO();
        //后续map加判断可加参数
        Map<String, String> map = new HashMap<>();
        map.put("接收事件", content);
        map.put("接收人", memberName);
        List<ItemDTO> items = getItemsList(map);
        List<String> userList = new ArrayList<>();
        userList.add(wxUserId);
        messageDTO.setAppid(config.getAppid());
        messageDTO.setUserIds(userList);
        messageDTO.setPage(pageUrl);
        messageDTO.setTitle(title);
        messageDTO.setItems(items);
        qywxSuiteApiService.sendMessage(wxEnterprise.getCorpid(), config.getWxSuiteid(), messageDTO);
    }

    @Override
    public String getNewWxUserId(String wxEnterpriseId, String wxUserId) {
        WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseService.selectById(wxEnterpriseId);
        String corpid = wxEnterpriseDTO.getCorpid();
        TabHaobanStaff tabHaobanStaff = staffService.selectByUserIdAndEnterpriseId(wxUserId, wxEnterpriseId);
        if (tabHaobanStaff == null) {
            return "";
        }
        return corpid.length() > 20 ? tabHaobanStaff.getWxOpenUseId() : tabHaobanStaff.getWxUserId();
    }


}
